@@ -36,6 +36,8 @@
 #include "common/utils.h"
 #include "cmds/commands.h"
 #include "common/help.h"
+#include "common/open-utils.h"
+#include "common/units.h"
 
 static int verbose = 0;
 static int no_pretty = 0;
@@ -254,7 +256,7 @@ static void print_seek_histogram(struct root_stats *stat)
 		if (group_count) {
 
 			gticks = group_count / tick_interval;
-			printf("\t\t%*Lu - %*Lu: %*Lu ", digits, group_start,
+			printf("\t\t%*llu - %*llu: %*llu ", digits, group_start,
 			       digits, group_end, digits, group_count);
 			if (gticks) {
 				for (i = 0; i < gticks; i++)
@@ -269,7 +271,7 @@ static void print_seek_histogram(struct root_stats *stat)
 		if (ticks <= 2)
 			continue;
 
-		printf("\t\t%*Lu - %*Lu: %*Lu ", digits, seek->distance,
+		printf("\t\t%*llu - %*llu: %*llu ", digits, seek->distance,
 		       digits, seek->distance, digits, seek->count);
 		for (i = 0; i < ticks; i++)
 			printf("#");
@@ -279,7 +281,7 @@ static void print_seek_histogram(struct root_stats *stat)
 		u64 gticks;
 
 		gticks = group_count / tick_interval;
-		printf("\t\t%*Lu - %*Lu: %*Lu ", digits, group_start,
+		printf("\t\t%*llu - %*llu: %*llu ", digits, group_start,
 		       digits, group_end, digits, group_count);
 		if (gticks) {
 			for (i = 0; i < gticks; i++)
