@@ -18,9 +18,9 @@
 #include "common/internal.h"
 #include "common/messages.h"
 #include "common/utils.h"
-#include "kernel-lib/crc32c.h"
+#include "crypto/crc32c.h"
 #include "image/sanitize.h"
-#include "extent_io.h"
+#include "kernel-shared/extent_io.h"
 
 /*
  * Reverse CRC-32C table
