@@ -4,13 +4,35 @@
 
 LANG=C
 SCRIPT_DIR=$(dirname $(readlink -f "$0"))
-TOP=$(readlink -f "$SCRIPT_DIR/../")
+if [ -z "$TOP" ]; then
+	TOP=$(readlink -f "$SCRIPT_DIR/../")
+	if [ -f "$TOP/configure.ac" ]; then
+		# inside git
+		TEST_TOP="$TOP/tests"
+		INTERNAL_BIN="$TOP"
+	else
+		# external, defaults to system binaries
+		TOP=$(dirname `which btrfs`)
+		TEST_TOP="$SCRIPT_DIR"
+		INTERNAL_BIN="$TEST_TOP"
+	fi
+else
+	# assume external, TOP set from commandline
+	TEST_TOP="$SCRIPT_DIR"
+	INTERNAL_BIN="$TEST_TOP"
+fi
+if ! [ -x "$TOP/btrfs" ]; then
+	echo "ERROR: cannot execute btrfs from TOP=$TOP"
+	exit 1
+fi
 TEST_DEV=${TEST_DEV:-}
-RESULTS="$TOP/tests/fsck-tests-results.txt"
-IMAGE="$TOP/tests/test.img"
+RESULTS="$TEST_TOP/fsck-tests-results.txt"
+IMAGE="$TEST_TOP/test.img"
 
-source "$TOP/tests/common"
+source "$TEST_TOP/common"
 
+export INTERNAL_BIN
+export TEST_TOP
 export TOP
 export RESULTS
 export LANG
@@ -32,7 +54,7 @@ run_one_test() {
 	testname="$1"
 	echo "    [TEST/fsck]   $(basename $testname)"
 	cd "$testname"
-	echo "=== Entering $testname" >> "$RESULTS"
+	echo "=== START TEST $testname" >> "$RESULTS"
 	if [ -x test.sh ]; then
 		# Type 2
 		./test.sh
@@ -42,11 +64,19 @@ run_one_test() {
 			fi
 			_fail "test failed for case $(basename $testname)"
 		fi
+		# These tests have overridden check_image() and their images may
+		# have intentional unaligned metadata to trigger subpage
+		# warnings (like fsck/018), skip the check for their subpage
+		# warnings.
+		#
+		# We care about subpage related warnings for write operations
+		# (mkfs/convert/repair), not those read-only checks on crafted
+		# images.
 	else
 		# Type 1
 		check_all_images
 	fi
-	cd "$TOP"
+	cd "$TEST_TOP"
 }
 
 # Each dir contains one type of error for btrfsck test.
@@ -62,7 +92,7 @@ run_one_test() {
 #    This is for case btrfs-image can't dump or case needs extra
 #    check/verify
 
-for i in $(find "$TOP/tests/fsck-tests" -maxdepth 1 -mindepth 1 -type d	\
+for i in $(find "$TEST_TOP/fsck-tests" -maxdepth 1 -mindepth 1 -type d	\
 	${TEST:+-name "$TEST"} | sort)
 do
 	run_one_test "$i"
