@@ -7,12 +7,9 @@ check_prereq btrfs
 
 # redefine the one provided by common
 check_image() {
-	local image
-
-	image=$1
-	run_mayfail $TOP/btrfs inspect-internal dump-tree "$image"
+	run_mayfail "$TOP/btrfs" inspect-internal dump-tree "$1"
 }
 
-check_all_images $TEST_TOP/fuzz-tests/images
+check_all_images "$TEST_TOP/fuzz-tests/images"
 
 exit 0
