@@ -20,10 +20,9 @@ test_uuid_random()
 
 	origuuid=11111111-a101-4031-b29a-379d4f8b7a2d
 
-	run_check $SUDO_HELPER "$TOP/mkfs.btrfs" -f \
+	run_check_mkfs_test_dev    \
 		--uuid "$origuuid" \
-		--rootdir "$INTERNAL_BIN/Documentation" \
-		"$TEST_DEV"
+		--rootdir "$INTERNAL_BIN/Documentation"
 	run_check "$TOP/btrfs" inspect-internal dump-super "$TEST_DEV"
 	currentfsid=$(run_check_stdout "$TOP/btrfstune" -f -u "$TEST_DEV" | \
 		grep -i 'current fsid:' | awk '{print $3}')
@@ -42,10 +41,9 @@ test_uuid_user()
 	origuuid=22222222-d324-4f92-80e9-7658bf3b845f
 	newuuid=33333333-bfc9-4045-9399-a396dc6893b3
 
-	run_check $SUDO_HELPER "$TOP/mkfs.btrfs" -f \
+	run_check_mkfs_test_dev    \
 		--uuid "$origuuid" \
-		--rootdir "$INTERNAL_BIN/Documentation" \
-		"$TEST_DEV"
+		--rootdir "$INTERNAL_BIN/Documentation"
 	run_check "$TOP/btrfs" inspect-internal dump-super "$TEST_DEV"
 	run_check "$TOP/btrfstune" -f -U "$newuuid" \
 		"$TEST_DEV"
