@@ -20,15 +20,9 @@ get_log_root_level() {
 test_zero_log()
 {
 	# FIXME: we need an image with existing log_root
-	run_check $SUDO_HELPER "$TOP/mkfs.btrfs" -f \
-		--rootdir "$INTERNAL_BIN/Documentation" \
-		"$TEST_DEV"
+	run_check_mkfs_test_dev --rootdir "$INTERNAL_BIN/Documentation"
 	run_check "$TOP/btrfs" inspect-internal dump-super "$TEST_DEV"
-	if [ "$1" = 'standalone' ]; then
-		run_check "$TOP/btrfs" rescue zero-log "$TEST_DEV"
-	else
-		run_check "$TOP/btrfs-zero-log" "$TEST_DEV"
-	fi
+	run_check "$TOP/btrfs" rescue zero-log "$TEST_DEV"
 	log_root=$(get_log_root "$TEST_DEV")
 	log_root_level=$(get_log_root "$TEST_DEV")
 	if [ "$log_root" != 0 ]; then
